/////////////////////////////////////////////////////////////////////////////////////////
//
// 1-Wire Library V2
//
// --------------------------------------------------------------------------------------
// 
// Filename:      1-wire.c
// Version:       2.0
// Date:          01/11/2011
// Author:        Joel Guittet - http://myfreescalewebpage.free.fr
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Description
//
// This files contains the functions used to communicate with the 1-Wire interface.
// This library is able to communicate with multiple devices connected to the
// microcontroller, each on a separated pin.
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Revisions
//
// Version	| Author		| Description
// --------------------------------------------------------------------------------------
// 2.0      | Joel Guittet  | Initial Release
//
/////////////////////////////////////////////////////////////////////////////////////////


//---------------------------------------------------------------------------------------
// Include
//---------------------------------------------------------------------------------------

#include "1-wire.h"


//---------------------------------------------------------------------------------------
// Name:        ONEWIRE_Init
// Param:		Interface: interface to initialize
// Return:      -
//
// Description:	Initialization of the 1-wire serial interface
//---------------------------------------------------------------------------------------
void ONEWIRE_Init(t_OneWireInterface * pxInterface)
{
	/* Set the pin High, 1-Wire device is powered throw the pull-up resistor */
	*pxInterface->PortData &= ~(0x01 << pxInterface->Bit);						/* PortData = 0 */
	*pxInterface->PortDataDirection &= ~(0x01 << pxInterface->Bit);				/* PortDataDirection = 0 */
}


//---------------------------------------------------------------------------------------
// Name:        ONEWIRE_Reset
// Param:		Interface: interface on which reset must be performed
// Return:      1-Wire reset result: NO_DEVICE_DETECTED, DEVICE_PRESENT or DEVICE_SHORTED
//
// Description:	Performs 1-Wire Reset
//---------------------------------------------------------------------------------------
t_OneWireResetResult ONEWIRE_Reset(t_OneWireInterface * pxInterface)
{
	BOOLEAN bShortDetect, bPresencePulseDetect;
		
	/* Reset Pulse: set the pin low, wait 500s and set the pin high */
	*pxInterface->PortData &= ~(0x01 << pxInterface->Bit);						/* PortData = 0 */
	*pxInterface->PortDataDirection |= (0x01 << pxInterface->Bit);  				/* PortDataDirection = 1 */
	ONEWIRE_DELAY(500)
	*pxInterface->PortData &= ~(0x01 << pxInterface->Bit);						/* PortData = 0 */
	*pxInterface->PortDataDirection &= ~(0x01 << pxInterface->Bit);				/* PortDataDirection = 0 */
	
	/* Short delay */
	ONEWIRE_DELAY(5)
	
	/* Short detect: if the 1-wire pin is low, it is because the 1-wire device is shorted */
	bShortDetect = (BOOLEAN)((*pxInterface->PortData & (0x01 << pxInterface->Bit)) == 0);
	
	/* Wait the device answer */
	ONEWIRE_DELAY(65)
	
	/* Presence pulse: if the 1-wire pin is low, it is because the 1-wire device is present (or shorted) */
	bPresencePulseDetect = (BOOLEAN)((*pxInterface->PortData & (0x01 << pxInterface->Bit)) == 0);
	
	/* Delay */
	ONEWIRE_DELAY(500)
	
	/* Result */
	if (bShortDetect == TRUE)
		return DEVICE_SHORTED;
	else if (bPresencePulseDetect == TRUE)
		return DEVICE_PRESENT;
	else
		return NO_DEVICE_DETECTED;
}
 

//---------------------------------------------------------------------------------------
// Name:        ONEWIRE_WriteByte
// Param:		Interface: interface on which byte must be written
//				Byte: byte which have to be written on the 1-wire interface
// Return:		-
//
// Description:	Write a byte
//---------------------------------------------------------------------------------------
void ONEWIRE_WriteByte(t_OneWireInterface * pxInterface, UINT8 u8Byte)
{
	UINT8 u8Index;
	
	/* Write the 8 bits on the 1-wire pin */
	for (u8Index = 0; u8Index < 8; u8Index++)
	{
		/* Pulse */
		*pxInterface->PortData &= ~(0x01 << pxInterface->Bit);					/* PortData = 0 */
		*pxInterface->PortDataDirection |= (0x01 << pxInterface->Bit);  		/* PortDataDirection = 1 */
	
		if ((u8Byte & 0x01 << u8Index) == 0)
		{
			/* '0' => Long Pulse */
			ONEWIRE_DELAY(60)
			*pxInterface->PortData &= ~(0x01 << pxInterface->Bit);				/* PortData = 0 */
			*pxInterface->PortDataDirection &= ~(0x01 << pxInterface->Bit);		/* PortDataDirection = 0 */
			ONEWIRE_DELAY(5)
		}
		else
		{
			/* '1' => Short Pulse */
			ONEWIRE_DELAY(5)
			*pxInterface->PortData &= ~(0x01 << pxInterface->Bit);				/* PortData = 0 */
			*pxInterface->PortDataDirection &= ~(0x01 << pxInterface->Bit);		/* PortDataDirection = 0 */
			ONEWIRE_DELAY(60)
		}
	}
}


//---------------------------------------------------------------------------------------
// Name:     	ONEWIRE_ReadByte
// Param:		Interface: interface on which byte must be read
// Return:      Byte read
//
// Description:	Read a byte
//---------------------------------------------------------------------------------------
UINT8 ONEWIRE_ReadByte(t_OneWireInterface * pxInterface)
{
	UINT8 u8Index, u8Byte = 0;
	
	/* Read the 8 bits */
	for (u8Index = 0; u8Index < 8; u8Index++)
	{
		/* Short Pulse */
		*pxInterface->PortData &= ~(0x01 << pxInterface->Bit);					/* PortData = 0 */
		*pxInterface->PortDataDirection |= (0x01 << pxInterface->Bit);  		/* PortDataDirection = 1 */
		ONEWIRE_DELAY(5)
		*pxInterface->PortData &= ~(0x01 << pxInterface->Bit);					/* PortData = 0 */
		*pxInterface->PortDataDirection &= ~(0x01 << pxInterface->Bit);			/* PortDataDirection = 0 */
	
		/* Delay */
		ONEWIRE_DELAY(5)    
	
		/* Read the 1-wire device response */
		if ((*pxInterface->PortData & (0x01 << pxInterface->Bit)) != 0) u8Byte += 0x01 << u8Index;    
	
		/* Delay */
		ONEWIRE_DELAY(55)
	}
	
	return u8Byte;
}


//---------------------------------------------------------------------------------------
// Name:        ONEWIRE_SetStrongPullUp
// Param:    	Interface: interface on which strong pull-up must be set
// Return:      -
//
// Description:	Set strong pull-up in order to power the 1-Wire device throw the microcontroller
//---------------------------------------------------------------------------------------
void ONEWIRE_SetStrongPullUp(t_OneWireInterface * pxInterface)
{
	/* Set Strong Pull-Up */
	*pxInterface->PortData |= (0x01 << pxInterface->Bit);						/* PortData = 1 */
	*pxInterface->PortDataDirection |= (0x01 << pxInterface->Bit);  			/* PortDataDirection = 1 */
}
