/////////////////////////////////////////////////////////////////////////////////////////
//
// 1-Wire Library V2 Project example
//
// --------------------------------------------------------------------------------------
//
// Filename:      main.c
// Version:       1.1
// Date:          31/07/2012
// Author:        Joel Guittet - http://myfreescalewebpage.free.fr
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Description
//
// This project example uses two DS18B20 temperature sensors from Dallas/Maxim Semiconductors
// to show a simple implementation of the 1-wire protocol.
// The 1-wire functions can be used with any 1-wire devices and any microcontrollers,
// there is no need except, of course, some GPIO to connect the 1-wire devices.
// The functions used in this projects example are able to manage multiple devices
// connected to the same microcontroller, each device on a separated pin.
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Revisions
//
// Version	| Author		| Description
// --------------------------------------------------------------------------------------
// 1.0      | Joel Guittet  | Initial Release
// 1.1      | Joel Guittet  | Coding improvements
//
/////////////////////////////////////////////////////////////////////////////////////////


//---------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------

#include <hidef.h>
#include "derivative.h"
#include "ds18b20.h"


//---------------------------------------------------------------------------------------
// Prototypes
//---------------------------------------------------------------------------------------

static void InitCPU(void);
static void InitMCG(void);
static void InitGPIO(void);


//---------------------------------------------------------------------------------------
// Name:        main
// Param:		-
// Return:      -
//
// Description:	Main function, never return
//---------------------------------------------------------------------------------------
void main(void)
{
	UINT16 u16Index;
	t_DS18B20Device xDS18B20Device1;
	t_DS18B20Device xDS18B20Device2;
		
	/* Initializations */
	InitCPU();
		
	/* Initialize the DS18B20 sensors, temperature resolution set to 9bits */
	(void)DS18B20_Init(&xDS18B20Device1, &PTED, &PTEDD, 3, 9);
	(void)DS18B20_Init(&xDS18B20Device2, &PTED, &PTEDD, 4, 9);

	EnableInterrupts;

	/* Main loop */
	for(;;)
	{
		/* Initiate a new temperature conversion on each device (take up to 94ms) */
		(void)DS18B20_ConvertT(&xDS18B20Device1);
		(void)DS18B20_ConvertT(&xDS18B20Device2);
		
		/* Make a small delay */
		for (u16Index = 0; u16Index < 65535; u16Index++);
		
		/* Now read the temperature on each device */
		(void)DS18B20_ReadT(&xDS18B20Device1);
		(void)DS18B20_ReadT(&xDS18B20Device2);
		
		/* Make a small delay */
		for (u16Index = 0; u16Index < 65535; u16Index++);

		__RESET_WATCHDOG();
	}
}


//---------------------------------------------------------------------------------------
// Name:        InitCPU
// Param:		-
// Return:      -
//
// Description:	Initialize CPU
//---------------------------------------------------------------------------------------
static void InitCPU(void)
{
	InitMCG();
	InitGPIO();
}


//---------------------------------------------------------------------------------------
// Name:        InitMCG
// Param:    	-
// Return:      -
//
// Description:	MCG configuration: Crystal=12MHz, Fbus = 18MHz
//---------------------------------------------------------------------------------------
static void InitMCG(void)
{
	MCGC2 = 0x36;
	while(MCGSC_OSCINIT != 1);
	MCGC1 = 0xB8;
	while(MCGSC_IREFST != 0);
	while(MCGSC_CLKST != 0b10);  
	MCGC1 = 0x98;
	MCGC3 = 0x46;
	while(MCGSC_PLLST != 1);
	while(MCGSC_LOCK != 1);
	MCGC1 = 0x18;
	while(MCGSC_CLKST != 0b11);
}


//---------------------------------------------------------------------------------------
// Name:        InitGPIO
// Param:    	-
// Return:      -
//
// Description:	IO ports initialization
//---------------------------------------------------------------------------------------
static void InitGPIO(void)
{
}
